package fr.mrdev023.vulkan_java.vk;

import fr.mrdev023.vulkan_java.vk.compatibilities.validators.PhysicalDeviceCompatibilityValidator;
import fr.mrdev023.vulkan_java.vk.utils.SuitablePhysicalDeviceFinder;
import fr.mrdev023.vulkan_java.window.Display;

import java.util.Optional;
import java.util.Set;

import org.lwjgl.vulkan.KHRDynamicRendering;
import org.lwjgl.vulkan.KHRSwapchain;
import org.lwjgl.vulkan.VK10;

import static org.lwjgl.glfw.GLFWVulkan.glfwVulkanSupported;

public class Vulkan {

    private static Instance instance;
    private static PhysicalDevice physicalDevice;
    private static Device device;
    private static Surface surface;

    public static void init() throws VulkanError {
        if (!glfwVulkanSupported()) {
            throw new IllegalStateException("Cannot find a compatible Vulkan installable client driver (ICD)");
        }

        instance = new Instance(true);
        surface = new Surface(instance, Display.getWindow());

        var physicalDeviceCompatibilityValidator = new PhysicalDeviceCompatibilityValidator(
            VK10.VK_QUEUE_GRAPHICS_BIT | VK10.VK_QUEUE_COMPUTE_BIT | VK10.VK_QUEUE_TRANSFER_BIT,
            Set.of(KHRDynamicRendering.VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME, KHRSwapchain.VK_KHR_SWAPCHAIN_EXTENSION_NAME),
            Optional.of(surface)
        );

        var physicalDeviceMatch = SuitablePhysicalDeviceFinder.findBestPhysicalDevice(instance, physicalDeviceCompatibilityValidator);
        if (physicalDeviceMatch == null) {
            throw new RuntimeException("No suitable physical device found");
        }

        physicalDevice = physicalDeviceMatch.physicalDevice;
        device = new Device(physicalDeviceMatch);
    }

    public static void destroy() {
        surface.destroy();
        device.destroy();
        physicalDevice.destroy();
        instance.destroy();
    }

}
