package fr.mrdev023.vulkan_java.vk.compatibilities.requirements;

import java.nio.IntBuffer;
import java.util.Optional;

import org.lwjgl.system.MemoryStack;
import org.lwjgl.vulkan.KHRSurface;
import org.lwjgl.vulkan.VK10;
import org.lwjgl.vulkan.VkQueueFamilyProperties;

import fr.mrdev023.vulkan_java.vk.PhysicalDevice;
import fr.mrdev023.vulkan_java.vk.Surface;

public class SurfacePresentQueueRequirement  implements IQueueFamilyRequirement {

    private final Surface surface;

    public SurfacePresentQueueRequirement(Surface surface) {
        this.surface = surface;
    }

    @Override
    public boolean isFamilySuitable(PhysicalDevice physicalDevice, VkQueueFamilyProperties queueFamilyProperties, int queueFamilyIndex) {
        try (MemoryStack stack = MemoryStack.stackPush()) {
            IntBuffer presentSupport = stack.ints(VK10.VK_FALSE);
            KHRSurface.vkGetPhysicalDeviceSurfaceSupportKHR(physicalDevice.getVkPhysicalDevice(), queueFamilyIndex, surface.getVkSurface(), presentSupport);
            return presentSupport.get(0) == VK10.VK_TRUE;
        }
    }

    @Override
    public String getName() {
        return "Surface Present Queue";
    }

    @Override
    public Optional<Integer> getQueueFlag() {
        return Optional.empty();
    }
}
